// MGShapesView.cpp : implementation of the CMGShapesView class
//

#include "stdafx.h"
#include "MGShapes.h"

#include "MGShapesDoc.h"
#include "MGShapesView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMGShapesView

IMPLEMENT_DYNCREATE(CMGShapesView, CView)

BEGIN_MESSAGE_MAP(CMGShapesView, CView)
	//{{AFX_MSG_MAP(CMGShapesView)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_ELLIPSE, OnEllipse)
	ON_UPDATE_COMMAND_UI(ID_ELLIPSE, OnUpdateEllipse)
	ON_COMMAND(ID_FREEHAND, OnFreehand)
	ON_UPDATE_COMMAND_UI(ID_FREEHAND, OnUpdateFreehand)
	ON_COMMAND(ID_RECTANGLE, OnRectangle)
	ON_UPDATE_COMMAND_UI(ID_RECTANGLE, OnUpdateRectangle)
	ON_WM_SIZE()
	ON_COMMAND(ID_DIAMOND, OnDiamond)
	ON_UPDATE_COMMAND_UI(ID_DIAMOND, OnUpdateDiamond)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_COMMAND(ID_TEXT, OnText)
	ON_UPDATE_COMMAND_UI(ID_TEXT, OnUpdateText)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMGShapesView construction/destruction

CMGShapesView::CMGShapesView()
{
	// TODO: add construction code here
	m_pShape = 0;
	TurnAllOffBut(m_FreeHand );
	m_brBackGround.CreateSolidBrush( RGB(255,255,255) );
}

CMGShapesView::~CMGShapesView()
{
}

BOOL CMGShapesView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMGShapesView drawing

void CMGShapesView::OnDraw(CDC* pDC)
{
	CMGShapesDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// MG: Do all drawing to the video buffer
	memDC.FillRect( &m_Rect, &m_brBackGround );

	int i=0;
	CShape* pShape;
	
	// MG: Iterate through shapes, and call their OnDraw functions
	while( (pShape = pDoc->GetShape(i++)) )
		pShape->OnDraw( &memDC );

	// MG: Draw the current shape
	if( m_pShape )
		m_pShape->OnDraw( &memDC );

	// MG: Then draw buffer onto the screen (avoid flicker)
	pDC->BitBlt( 0, 0, m_Rect.right, m_Rect.bottom, &memDC, 0, 0, SRCCOPY );
}

/////////////////////////////////////////////////////////////////////////////
// CMGShapesView printing

BOOL CMGShapesView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CMGShapesView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CMGShapesView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CMGShapesView diagnostics

#ifdef _DEBUG
void CMGShapesView::AssertValid() const
{
	CView::AssertValid();
}

void CMGShapesView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMGShapesDoc* CMGShapesView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMGShapesDoc)));
	return (CMGShapesDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMGShapesView message handlers

void CMGShapesView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// MG: Dynamically create the Shape object
	if( m_FreeHand )
		m_pShape = new CShapeFreeHand;
	else if( m_Ellipse )
		m_pShape = new CShapeEllipse;
	else if( m_Diamond )
		m_pShape = new CShapeDiamond;
	else if( m_Text )
		m_pShape = new CShapeText;
	else
		m_pShape = new CShapeRect;

	// MG: Start using the m_pShape, not caring about shape type
	m_pShape->StartPoint( point );

	CView::OnLButtonDown(nFlags, point);
}

void CMGShapesView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if( m_pShape )
	{
		// MG: If we have a shape, add mid point to it 
		m_pShape->MidPoint( point );
		InvalidateRect(0, FALSE);
	}
	CView::OnMouseMove(nFlags, point);
}

void CMGShapesView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if( m_pShape )
	{

		CMGShapesDoc* pDoc = GetDocument();
		m_pShape->EndPoint( point );

		// MG: Store shape to document, and force window to redraw.
		pDoc->AddShape( m_pShape );
		InvalidateRect( 0, FALSE);
		m_pShape = 0;
	}
	
	CView::OnLButtonUp(nFlags, point);
}

void CMGShapesView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	// MG: Release our video buffer
	if( memBitmap.GetSafeHandle() )
	{
		memDC.DeleteDC();
		memBitmap.DeleteObject();
	}

	// MG: Create our bideo buffer
	memDC.CreateCompatibleDC(0);
	memBitmap.CreateCompatibleBitmap( &memDC, cx, cy );
	memDC.SelectObject( &memBitmap );
	m_Rect.SetRect(0, 0, cx, cy );	
}

void CMGShapesView::TurnAllOffBut(bool & Dest)
{
	// MG: Helper routine, centralizes flag setting and claering.
	m_Text=m_FreeHand=m_Ellipse=m_Rectangle=m_Diamond=false;
	Dest = true;
}

// MG: Menu handlers and UI updates
void CMGShapesView::OnEditUndo() 
{
	CMGShapesDoc* pDoc = GetDocument();
	pDoc->RemoveShape();
	InvalidateRect( 0, FALSE );	
}

void CMGShapesView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	CMGShapesDoc* pDoc = GetDocument();
	pCmdUI->Enable( pDoc->GetShapeCount() > 0 ? TRUE : FALSE );
}

void CMGShapesView::OnEllipse() 
{
	TurnAllOffBut(m_Ellipse);
}

void CMGShapesView::OnUpdateEllipse(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_Ellipse ? 1 : 0 );	
}

void CMGShapesView::OnFreehand() 
{
	TurnAllOffBut(m_FreeHand);
}

void CMGShapesView::OnUpdateFreehand(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_FreeHand ? 1 : 0 );	
}

void CMGShapesView::OnRectangle() 
{
	TurnAllOffBut(m_Rectangle);
}

void CMGShapesView::OnUpdateRectangle(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_Rectangle ? 1 : 0 );	
}

void CMGShapesView::OnDiamond() 
{
	TurnAllOffBut(m_Diamond);
}

void CMGShapesView::OnUpdateDiamond(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_Diamond ? 1 : 0 );	
}

void CMGShapesView::OnText() 
{
	TurnAllOffBut( m_Text );	
}

void CMGShapesView::OnUpdateText(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_Text ? 1 : 0 );	
	
}
