// MStatic.cpp : implementation file
//

#include "stdafx.h"
#include "MStatic.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// MG: This class was imported into this project.  It is an extended CStatic
// class, that has the ability to control appearance a little easier, as well
// as act like a 'link' where it will execute a program when clicked (like
// a URL link on a web page).
/////////////////////////////////////////////////////////////////////////////
// CMStatic

CMStatic::CMStatic()
{
	hCursor = 0;		
	bForeColor = bSharedCursor = false;
	bNoExecute = true;
}

CMStatic::~CMStatic()
{
	DestroyCursor();
}


BEGIN_MESSAGE_MAP(CMStatic, CStatic)
	//{{AFX_MSG_MAP(CMStatic)
	ON_WM_SETCURSOR()
	ON_CONTROL_REFLECT(BN_CLICKED, OnClicked)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMStatic message handlers

// OnSetCursor - Called by framework when mouse moves over the control
BOOL CMStatic::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if( hCursor )
	{
		::SetCursor( hCursor );
		return( TRUE );
	}
	return CStatic::OnSetCursor(pWnd, nHitTest, message);
}

// LoadCursor - Helper function - Loads cursor via its unique ID
// Parameters:	ID = ID of cursor to load, from the resource script
// Returns:		TRUE if succesful, FALSE upon error
// Comments:	Calls LoadCursor(LPCSTR lpszResource) function, setting cursor 
//				for the static control
bool CMStatic::LoadCursor(UINT ID)
{
	return( LoadCursor( MAKEINTRESOURCE(ID) ) );
}

// LoadCursor - Loads cursor via its resource name
// Parameters:	ID = ID of cursor to load, from the resource script
// Returns:		TRUE if succesful, FALSE upon error
// Comments:	Sets cursor for the static control
bool CMStatic::LoadCursor(LPCSTR lpszResource)
{
	DestroyCursor();

	if( !(hCursor = AfxGetApp()->LoadCursor( lpszResource )) )
	{
		if( !(hCursor = ::LoadCursor( NULL, lpszResource )) )
		{
			bSharedCursor = true;
			return( true );
		}
		bSharedCursor = false;
		return( false );
	}
	else
	{
		bSharedCursor = false;
		return( true );
	}
}

// DestroyCursor- Unloads Cursor, if one was loaded
void CMStatic::DestroyCursor()
{
	if( hCursor && !bSharedCursor )
		::DestroyCursor( hCursor );
	hCursor = 0;
	bSharedCursor = false;
}


// SetFontAttr - Sets various font attributes for the static control
// Parameters:	Fontname is the name of the new font, null to leave unchanged.
//				Height is the new height for the font, or -1 to leave unchanged
//				Width is the new width for the font, or -1 to leave unchanged
//				Bold is 0 to turn off, 1 to turn on, or -1 to leave unchanged
//				Italic is 0 to turn off, 1 to turn on, or -1 to leave unchanged
//				Underline is 0 to turn off, 1 to turn on, or -1 to leave unchanged
//				Fore is 0 to not use FontColor, 1 to use it, or -1 to leave unchanged
//				FontColor is new font color (based on 'Fore' parameter).
// Comments:	Sets cursor for the static control
void CMStatic::SetFontAttr(const char *Fontname, int Height, int Width, int Bold, int Italic, int Under, int Fore, COLORREF FontClr )
{
	CFont *Font;
	LOGFONT lf;

	Font = GetFont();
	if( Font->GetObject( sizeof( lf ), &lf ) )
	{
		if( Fontname )
		{
			memset( lf.lfFaceName, 0, sizeof(lf.lfFaceName) );
			strncpy( lf.lfFaceName, Fontname, sizeof(lf.lfFaceName) );
		}
		if( Height!=-1 )
			lf.lfHeight = Height;
		if( Width!=-1 )
			lf.lfWidth = Width;
		if( Bold != -1 )
			lf.lfWeight = Bold ? FW_BOLD : FW_NORMAL;
		if( Italic != -1 )
			lf.lfItalic = Italic ? TRUE : FALSE;
		if( Under != -1 )
			lf.lfUnderline = Under ? TRUE : FALSE;
		if( cFont.GetSafeHandle() )
			cFont.DeleteObject();
		cFont.CreateFontIndirect( & lf );
		SetFont( &cFont );
		
		bBold = lf.lfWeight==FW_BOLD;
		bItalic = lf.lfItalic==TRUE;
		bUnderlined = lf.lfUnderline==TRUE;
	}
	if( Fore != -1 )
	{
		if( Fore )
		{
			bForeColor = true;
			crForeColor = FontClr;
		}
		else
			bForeColor = false;

	}

}

// SetBold - Helper function to turn Bold font on or off
// Parameters:	Mode: true to set to bold, false to set to normal
// Returns:		Previous setting
bool CMStatic::SetBold(bool Mode)
{
	bool Ret = bBold;
	if( bBold != Mode )
		SetFontAttr( NULL, -1, -1, Mode?1:0);
	return( Ret );
}

// SetItalic - Helper function to turn Italics font on or off
// Parameters:	Mode: true to set to italic, false to set to normal
// Returns:		Previous setting
bool CMStatic::SetItalic(bool Mode)
{
	bool Ret = bItalic;
	if( bItalic != Mode )
		SetFontAttr( NULL, -1, -1, -1, Mode?1:0);
	return( Ret );
}

// SetUnderline - Helper function to turn Underline font on or off
// Parameters:	Mode: true to set to underline, false to set to normal
// Returns:		Previous setting
bool CMStatic::SetUnderline(bool Mode)
{
	bool Ret = bUnderlined;
	if( bUnderlined != Mode )
		SetFontAttr( NULL, -1, -1, -1, -1, Mode?1:0);
	return( Ret );
}

// SetForeColor - Helper function set text color
// Parameters:	Mode: true to set color on, false to use defualt
//				Color: New color, if Mode is true
void CMStatic::SetForeColor( bool Mode, COLORREF Color )
{
	SetFontAttr( NULL, -1, -1, -1, -1, -1, Mode?1:0, Color );
}


// OnClicked - Indicates user clicked the control
// Comments:	If the bNoExecute parameter is false, then the 'Command'
//				is executed
void CMStatic::OnClicked() 
{
	if( ! bNoExecute )
	{
		if( !Command.IsEmpty() ) 
			ShellExecute( *GetParent(), "OPEN", Command, "", "", SW_NORMAL );
		else
		{
			CString Tmp;
			GetWindowText( Tmp );
			ShellExecute( *GetParent(), "OPEN", Tmp, "", "", SW_NORMAL );
		}
	}
	else
	{
		// If this window had SS_NOTIFY, then send the BN_CLICKED off to the 
		// Message Map of the main frame, as if a normal windows message.
		if( GetWindowLong( *this, GWL_STYLE ) & SS_NOTIFY )
			AfxGetMainWnd()->OnCmdMsg( GetDlgCtrlID(), BN_CLICKED, 0, 0 );
	}
}

// SetText - Sets the window text, and the 'Command' option
// Comments:	If the bNoExecute parameter is false, then the 'Command'
//				is executed when the user clicks the control
void CMStatic::SetText(const char *lpszCaption, const char *lpszCommand)
{
	if( lpszCaption )
		SetWindowText( lpszCaption );
	Command = lpszCommand?lpszCommand:"";
}

// SizeFontToFit- Helper function to size the font to fill the window size
// Parameters:	Mode is 0 to perform sizing based on height only
//				1 to perform sizing based on width only
//				2 to perform sizing based on height and width
void CMStatic::SizeFontToFit( int Mode )
{
	CRect Rect;
	GetClientRect( &Rect );
	if( Mode == 0 )
		SetFontAttr( 0, Rect.bottom, 0 );
	if( Mode == 1 )
		SetFontAttr( 0, 0, Rect.right  );
	if( Mode == 2 )
		SetFontAttr( 0, Rect.bottom, Rect.right  );
}

// SizeRectToFit- Sizes the child static window to fit the current font setting
void CMStatic::SizeRectToFit()
{
	CDC* pDC = GetDC();
	
	CString Tmp;
	GetWindowText( Tmp );
	pDC->SelectObject( GetFont() );
	CSize Size = pDC->GetTextExtent( Tmp );
	SetWindowPos( 0, 0, 0, Size.cx, Size.cy , SWP_NOMOVE|SWP_NOZORDER );

	ReleaseDC( pDC );
}

// SetURL- Helper function to setup static control to reflect a URL link
// Parameters:	lpszCaption: Text to display in window
//				lpszURL: URL to go to upon click
// Comments:	Sets text color, cursor, and underline to look like a link
void CMStatic::SetURL(const char *lpszCaption, const char *lpszURL)
{
	SetText( lpszCaption, lpszURL );
	SetForeColor( true, RGB( 0, 0, 255 ) );
	SetUnderline( true );
	UseHandCursor( true );
	bNoExecute = false;
}

// OnPaint- Handles display of the static control, called by framework.
void CMStatic::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	CString Tmp;
	GetWindowText( Tmp );

	dc.SelectObject( GetFont() );
	dc.SetBkMode( TRANSPARENT );
	if( bForeColor )
		dc.SetTextColor( crForeColor );
	dc.TextOut( 0, 0, Tmp );

}

/* This data created with Cursor Editor utility */

/* Don't forget that you may need to add the following prototype to a .h file: */

HCURSOR CreateMhandCursor(); /* Prototype */

unsigned char MhandCursorAndBits[]={
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x8F, 0xFF, 0xFF, 0xFF, 0x8F, 0xFF, 0xFF,
	0xFF, 0x07, 0xFF, 0xFF, 0xFF, 0x07, 0xFF, 0xFF, 0xFF, 0x07, 0xFF, 0xFF,
	0xFF, 0x07, 0xFF, 0xFF, 0xFF, 0x07, 0xFF, 0xFF, 0xFF, 0x00, 0xFF, 0xFF,
	0xFF, 0x00, 0x0F, 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xFF,
	0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0x83, 0x00, 0x00, 0xFF,
	0x81, 0x00, 0x00, 0xFF, 0x80, 0x00, 0x00, 0xFF, 0x80, 0x00, 0x00, 0xFF,
	0xE0, 0x00, 0x00, 0xFF, 0xF0, 0x00, 0x00, 0xFF, 0xF8, 0x00, 0x00, 0xFF,
	0xFC, 0x00, 0x00, 0xFF, 0xFE, 0x00, 0x01, 0xFF, 0xFE, 0x00, 0x01, 0xFF,
	0xFF, 0x00, 0x03, 0xFF, 0xFF, 0x00, 0x03, 0xFF, 0xFF, 0x80, 0x03, 0xFF,
	0xFF, 0x80, 0x03, 0xFF, 0xFF, 0x80, 0x03, 0xFF
};

unsigned char MhandCursorXorBits[]={

	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00,
	0x00, 0x70, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00,
	0x00, 0x70, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00,
	0x00, 0x77, 0x00, 0x00, 0x00, 0x77, 0x70, 0x00, 0x00, 0x77, 0x76, 0x00,
	0x00, 0x77, 0xF6, 0x00, 0x00, 0x7F, 0xFE, 0x00, 0x00, 0x7F, 0xFE, 0x00,
	0x38, 0x7F, 0xFE, 0x00, 0x3C, 0x7F, 0xFE, 0x00, 0x1E, 0x7F, 0xFE, 0x00,
	0x0F, 0xFF, 0xFE, 0x00, 0x07, 0xFF, 0xFE, 0x00, 0x03, 0xFF, 0xFE, 0x00,
	0x01, 0xFF, 0xFC, 0x00, 0x00, 0xFF, 0xFC, 0x00, 0x00, 0x7F, 0xFC, 0x00,
	0x00, 0x7F, 0xF8, 0x00, 0x00, 0x3F, 0xF8, 0x00, 0x00, 0x1F, 0xF8, 0x00,
	0x00, 0x1F, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00
};

int MhandCursorHotSpotX=10, MhandCursorHotSpotY=5;

HCURSOR CreateMhandCursor()
{
	return( CreateCursor( GetModuleHandle(NULL), MhandCursorHotSpotX, MhandCursorHotSpotY, 32, 32,
		MhandCursorAndBits, MhandCursorXorBits) );
}


// UseHandCursor- Helper function to set static control to use a hand cursor
// Parameters:	Mode: true to use a hand cursor when mouse moves over window, false to use default cursor.
void CMStatic::UseHandCursor(bool Mode)
{
	DestroyCursor();

	if( Mode )
	{
		// Note: Windows 2000 and many latest patches include a Hand cursor.  To be more more portable,
		// we will create our own hand cursor.
		bSharedCursor = true;
		hCursor = CreateMhandCursor();
	}
}

